#define STR_ENDGAME1	"We owe you a great debt, Mr. Blazkowicz."
#define STR_ENDGAME2	"You have served your country well."
#define STR_ENDGAME3	"With the spear gone, the Allies will finally"
#define STR_ENDGAME4	"by able to destroy Hitler..."

#define STR_COPY1		"That's close, but not close enough to get"
#define STR_COPY2		"you into the game."

#define STR_COPY3		"Wow, you must have the early version of the"
#define STR_COPY4		"manual with the totally false information in it."

#define STR_COPY5		"I would let you into the game, but seeing"
#define STR_COPY6		"as that was not at all the right answer..."

#define STR_COPY7		"It's just too bad we can't get together on"
#define STR_COPY8		"this one.  Sorry."

#define STR_COPY9		"Hey, you're just SO off base!"

#define STR_COPY10		"You know, I once typed that myself when"
#define STR_COPY11		"I was your age."

#define STR_COPY12		"Nops.  Zero points.  Zugga."

#define STR_COPY13		"Yeah...right."

#define STR_COPY14		"You must like these quizzes."

#define STR_COPY15		"Could be called \"PixelMeister\"?"

#define STR_COPY16		"Might engineer some software?"

#define STR_COPY17		"Would be found"
#define STR_COPY18		"directing creatively?"

#define STR_COPY19		"Might be found"
#define STR_COPY20		"handling operations?"

#define STR_COPY21		"Has a name familiar"
#define STR_COPY22		"to your weatherman?"

#define STR_NOPE1		"Welcome to the DOS prompt, pirate!"
#define STR_NOPE2		"Eat hot DOS prompt, goober!"
#define STR_NOPE3		"Ya know, this program doesn't cost that much."
#define STR_NOPE4		"Hey...weren't you just AT this DOS prompt?"
#define STR_NOPE5		"What's a nice user like you doin' at a DOS prompt like this?"
#define STR_NOPE6		"Well, I'm sure you just \"misplaced\" the manual..."
#define STR_NOPE7		"Run me again when you've boned up on your manual a bit."
#define STR_NOPE8		"Nice try, but no Spear."
#define STR_NOPE9		"That information is in the Spear of Destiny manual, by the way."

#define STR_MISC1		"Under \"Killing the Enemy\", what"
#define STR_MISC2		"type of enemy is pictured?"

#define STR_MISC3		"How many eyelets are on B.J.'s"
#define STR_MISC4		"boots? (see page 2)"

#define STR_MISC5		"The word \"minister\" appears in"
#define STR_MISC6		"what gray shape on page 2?"

#define STR_MISC7		"How many bullets does B.J. have"
#define STR_MISC8		"on the screen-shot in the middle"
#define STR_MISC9		"of page 9?"

#define STR_STAR	    "star"
#define STR_DEBRIEF	    " DEBRIEFING\n SESSION\n"
#define STR_ENEMY1	    "Name the member of the"
#define STR_ENEMY2	    "enemy forces shown above"

#define STR_CHECKMAN	"CHECK YER MANUAL!"
#define STR_MAN1	    "Which manual page"
#define STR_MAN2	    "is the Options Menu"
#define STR_MAN3	    "function"
#define STR_MAN4	    "on?"

#define STR_ID1		    "Which member of Id Software:"
